from __future__ import annotations

import itertools
import logging
from datetime import datetime, timezone
from io import StringIO
from uuid import uuid4

from license_expression import get_spdx_licensing
from spdx_tools.spdx.model.actor import Actor, ActorType
from spdx_tools.spdx.model.document import Document, CreationInfo
from spdx_tools.spdx.model.package import Package
from spdx_tools.spdx.model.spdx_none import SpdxNone
from spdx_tools.spdx.writer.tagvalue.tagvalue_writer import write_document

from checkov.common.output.extra_resource import ExtraResource
from checkov.common.output.record import SCA_PACKAGE_SCAN_CHECK_NAME, Record
from checkov.common.output.cyclonedx_consts import SCA_CHECKTYPES
from checkov.common.output.report import Report
from checkov.version import version

DOCUMENT_NAME = "checkov-sbom"
SPDXREF = "SPDXRef-"


class SPDX:
    def __init__(self, repo_id: str | None, reports: list[Report]):
        self.repo_id = f"{repo_id}/" if repo_id else ""
        self.reports = reports

        self.document = self.create_document()
        self.packages: list[Package] = []

        # each entry looks like '{file_name}#{package_name}#{package_version}'
        self._added_packages_cache: set[str] = set()

    def create_document(self) -> Document:
        creation_info = CreationInfo(
            spdx_version="SPDX-2.3",
            spdx_id="SPDXRef-DOCUMENT",
            name=DOCUMENT_NAME,
            data_license="CC0-1.0",
            document_namespace=f"https://spdx.org/spdxdocs/{DOCUMENT_NAME}-{version}-{uuid4()}",
            creators=[
                Actor(ActorType.TOOL, "checkov"),
                Actor(ActorType.ORGANIZATION, "bridgecrew", "meet@bridgecrew.io"),
            ],
            created=datetime.now(timezone.utc),
        )
        return Document(creation_info=creation_info)

    def get_tag_value_output(self) -> str:
        output = StringIO()

        self.add_packages_to_doc()
        write_document(document=self.document, text_output=output)

        return output.getvalue()

    def validate_licenses(self, package: Package, license_: str) -> None:
        if license_ and license_ not in ["Unknown license", "NOT_FOUND", "Unknown"]:
            split_licenses = license_.split(",")
            licenses = []

            for lic in split_licenses:
                lic = lic.strip('"')
                try:
                    licenses.append(get_spdx_licensing().parse(lic))
                except Exception as e:
                    logging.info(f"error occured when trying to parse the license:{split_licenses} due to error {e}")
            package.license_info_from_files = licenses

    def create_package(self, check: Record | ExtraResource) -> Package:
        package_data = check.vulnerability_details
        if not package_data:
            # this shouldn't happen
            logging.error(f"Check {check.resource} doesn't have 'vulnerability_details' set")
            return Package(name="unknown", spdx_id=f"{SPDXREF}unknown", download_location=SpdxNone())

        package_name = package_data.get("package_name")
        if not package_name:
            # this shouldn't happen
            logging.error(f"Package {package_data} doesn't have 'package_name' set")
            return Package(name="unknown", spdx_id=f"{SPDXREF}unknown", download_location=SpdxNone())

        package = Package(
            name=package_name,
            spdx_id=f"{SPDXREF}{package_name}",
            version=package_data["package_version"],
            download_location=SpdxNone(),
            file_name=check.file_path,
        )
        license_ = package_data.get("licenses", "")
        self.validate_licenses(package=package, license_=license_)

        return package

    def add_packages_to_doc(self) -> None:
        for report in self.reports:
            for check in itertools.chain(report.passed_checks, report.skipped_checks):
                if report.check_type in SCA_CHECKTYPES and check.check_name != SCA_PACKAGE_SCAN_CHECK_NAME:
                    continue
                package = self.create_package(check)
                self.add_new_package_to_list(package)

            for check in report.failed_checks:
                if report.check_type in SCA_CHECKTYPES and check.check_name != SCA_PACKAGE_SCAN_CHECK_NAME:
                    continue
                package = self.create_package(check)
                self.add_new_package_to_list(package)

            for resource in sorted(report.extra_resources):
                package = self.create_package(resource)
                self.add_new_package_to_list(package)

        if self.packages:
            self.document.packages = self.packages

    def add_new_package_to_list(self, package: Package) -> None:
        """Adds a package to the list, if it not exists"""

        package_cache_entry = f"{package.file_name}#{package.name}#{package.version}"
        if package_cache_entry not in self._added_packages_cache:
            self.packages.append(package)
            self._added_packages_cache.add(package_cache_entry)



from __future__ import annotations

from typing import Any

from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckResult, CheckCategories


class PolicyNoServiceAny(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure security rules do not have 'services' set to 'any' "
        id = "CKV_PAN_6"
        supported_resources = ('panos_security_policy', 'panos_security_rule_group')
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        # Check there is a rule defined in the resource
        if 'rule' in conf:

            # Report the area of evaluation
            self.evaluated_keys = ['rule']

            # Get all the rules defined in the resource
            rules = conf['rule']

            # Iterate over each rule
            for secrule in rules:

                # Check if services is defined in the resource
                if 'services' in secrule:

                    # If services is defined, get the value
                    services = secrule['services']

                    # The value "any" is overly permissive and is therefore a fail. The value "any" can only appear on its on, "any" with any other values in the list is rejected by Terraform during apply stage
                    if services[0][0] == "any":
                        return CheckResult.FAILED
                    # Any non-any value is specifying an service, which is a pass
                else:
                    # If "services" attribute is not defined, this is not valid and will fail during Terraform plan stage, and should therefore be a fail
                    return CheckResult.FAILED

            # No "any" found for the "services" attribute for any rules, therefore we have a pass
            return CheckResult.PASSED

        # If there's no rules we have nothing to check
        return CheckResult.UNKNOWN


check = PolicyNoServiceAny()



from __future__ import annotations
from abc import abstractmethod
from typing import Dict, Any

from checkov.common.output.record import Record
from checkov.policies_3d.checks_infra.base_check import Base3dPolicyCheck
import json


class Base3dPolicyCheckParser:
    def __init__(self, raw_check: dict[str, Any] | None = None):
        if not raw_check:
            return

        self.raw_check = raw_check

        check_code = json.loads(raw_check.get('code', ''))
        self.schema_version = check_code.get('version')
        self.check_definition = check_code.get('definition')

    @abstractmethod
    def parse_raw_check(self, raw_check: Dict[str, Dict[str, Any]], **kwargs: Any) -> Base3dPolicyCheck:
        pass

    def _fill_check_metadata(self, check: Base3dPolicyCheck) -> None:
        check.id = self.raw_check.get("id", "")
        check.name = self.raw_check.get("name", "")
        check.category = self.raw_check.get("category", "")
        check.guideline = self.raw_check.get("guideline", "")

    def parse(self, iac_records: list[Record] | None = None,
              secrets_records: list[Record] | None = None,
              cves_reports: list[dict[str, Any]] | None = None) -> Base3dPolicyCheck | None:
        if self.schema_version == 'v1':
            return self._parse_check_v1(iac_records or [], secrets_records or [], cves_reports or [])

        return None

    @abstractmethod
    def _parse_check_v1(self, iac_records: list[Record], secrets_records: list[Record], cves_reports: list[dict[str, Any]]) -> Base3dPolicyCheck:
        pass



from __future__ import annotations

from typing import Any

from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckResult, CheckCategories


class LambdaEnvironmentEncryptionSettings(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Check encryption settings for Lambda environmental variable"
        id = "CKV_AWS_173"
        supported_resources = ("aws_lambda_function",)
        categories = (CheckCategories.ENCRYPTION,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        # check that if I have env vars I have a KMS key
        if len(conf.get("environment", [])):
            if "kms_key_arn" in conf:
                if conf["kms_key_arn"] == [""]:
                    self.evaluated_keys = ["environment/kms_key_arn"]
                    return CheckResult.FAILED
                return CheckResult.PASSED
            self.evaluated_keys = ["environment"]
            return CheckResult.FAILED

        # no env vars so should be no key as that causes state mismatch
        if "kms_key_arn" in conf:
            if not len(conf["kms_key_arn"]):
                return CheckResult.PASSED
            return CheckResult.FAILED
        # neither env vars nor kms key
        return CheckResult.UNKNOWN

    def get_evaluated_keys(self) -> list[str]:
        return ["environment/[0]/variables"]


check = LambdaEnvironmentEncryptionSettings()



from checkov.common.models.consts import ANY_VALUE
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories


class GoogleIAMWorkloadIdentityConditional(BaseResourceValueCheck):
    def __init__(self):
        """
        A configuration for an external workload identity pool provider should have conditions set.
        In GitHub Actions, one can authenticate to Google Cloud by setting values for workload_identity_provider and service_account and requesting a short-lived OIDC token which is then used to execute commands as that Service Account. If you don't specify a condition in the workload identity provider pool configuration, then any GitHub Action can assume this role and act as that Service Account.
        This can be checked in Terraform configs by looking at whether a google_iam_workload_identity_pool_provider has the attribute_condition field set.

        Link: https://www.revblock.dev/exploiting-misconfigured-google-cloud-service-accounts-from-github-actions/
        """

        name = "Ensure IAM workload identity pool provider is restricted"
        id = "CKV_GCP_118"
        supported_resources = ['google_iam_workload_identity_pool_provider']
        categories = [CheckCategories.IAM]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return 'attribute_condition'

    def get_expected_value(self):
        return ANY_VALUE


check = GoogleIAMWorkloadIdentityConditional()



from __future__ import annotations

import logging
from enum import Enum
from typing import Dict, Any

from checkov.common.util.type_forcers import force_list

from checkov.common.output.record import Record
from checkov.policies_3d.checks_infra.base_parser import Base3dPolicyCheckParser
from checkov.policies_3d.checks_infra.base_check import Base3dPolicyCheck
from checkov.policies_3d.syntax.cves_syntax import RiskFactorCVEContains
from checkov.policies_3d.syntax.iac_syntax import ViolationIdEquals
from checkov.policies_3d.syntax.syntax import Predicament, Predicate
import itertools

SUPPORTED_LOGICAL_OPERATORS = ['and', 'or']
SCA_CHECK_ID_PREFIXES = ['CKV_CVE_', 'BC_LIC_1', 'BC_LIC_2']
SECRETS_CHECK_ID_PREFIXES = ['BC_GIT_']


class PredicateAttributes(str, Enum):
    RISK_FACTOR = 'risk_factor'
    VIOLATION_ID = 'violation_id'


class Policy3dParser(Base3dPolicyCheckParser):
    def __init__(self, raw_check: dict[str, Any] | None = None,
                 resource: str | None = None, records: list[Record] | None = None):
        super().__init__(raw_check)
        self.resource = resource
        self.records = records

    def parse_raw_check(self, raw_check: Dict[str, Dict[str, Any]], **kwargs: Any) -> Base3dPolicyCheck:
        """
        Deprecated. used for the first version of 3D policies
        """
        policy_definition = raw_check.get("definition", {})
        check = Base3dPolicyCheck()
        check.iac = policy_definition.get('iac', {})
        check.cve = policy_definition.get('cve', {})
        check.id = raw_check.get("metadata", {}).get("id", "")
        check.name = raw_check.get("metadata", {}).get("name", "")
        check.category = raw_check.get("metadata", {}).get("category", "")
        check.guideline = raw_check.get("metadata", {}).get("guideline")

        return check

    def _parse_check_v1(self, iac_records: list[Record], secrets_records: list[Record], cves_reports: list[dict[str, Any]]) -> Base3dPolicyCheck:
        check = Base3dPolicyCheck()
        self._fill_check_metadata(check)
        check.predicaments = []

        cves_definition: dict[str, Any] = {}
        iac_definition: dict[str, Any] = {}
        secrets_definition: dict[str, Any] = {}

        for definition in self.check_definition:
            if "cves" in definition.keys():
                cves_definition = definition["cves"]
            elif "iac" in definition.keys():
                iac_definition = definition["iac"]
            elif "secrets" in definition.keys():
                secrets_definition = definition["secrets"]

        cve_predicaments: list[Predicament] = list(filter(None, [self._create_module_predicament(cves_definition, cve_report) for cve_report in cves_reports]))
        iac_predicaments: list[Predicament] = list(filter(None, [self._create_module_predicament(iac_definition, iac_record) for iac_record in iac_records]))
        secrets_predicaments: list[Predicament] = list(filter(None, [self._create_module_predicament(secrets_definition, secrets_record) for secrets_record in secrets_records]))

        # Generating all predicaments combinations while filtering empty lists
        all_combinations = list(itertools.product(*filter(bool, [cve_predicaments, iac_predicaments, secrets_predicaments])))

        for combination in all_combinations:
            check.predicaments.append(
                Predicament(
                    logical_op='and',
                    predicaments=[predicament for predicament in combination]
                )
            )

        return check

    @staticmethod
    def _create_predicate(key: str, value: Any, record: Record | dict[str, Any]) -> Predicate | None:
        if isinstance(record, dict):
            # Specific to cve records passed as dicts
            if key == PredicateAttributes.RISK_FACTOR:
                return RiskFactorCVEContains(force_list(value), record)
        elif isinstance(record, Record):
            if key == PredicateAttributes.VIOLATION_ID:
                return ViolationIdEquals(record, value)

        logging.debug(f"Unable to create predicate for unsupported key {key}")
        return None

    def _create_module_predicament(self, policy_definition: dict[str, Any], record: Record | dict[str, Any]) -> Predicament | None:
        if not policy_definition:
            return None

        top_level_logical_op = ''
        if not any(op in policy_definition.keys() for op in SUPPORTED_LOGICAL_OPERATORS):
            top_level_logical_op = 'and'

        top_level_predicament = Predicament(logical_op=top_level_logical_op)

        for key, value in policy_definition.items():
            if key in SUPPORTED_LOGICAL_OPERATORS:
                top_level_logical_op = key
                top_level_predicament.logical_op = key
            else:
                predicate = self._create_predicate(key, value, record)
                if predicate:
                    top_level_predicament.predicates.append(predicate)

        nested_definition = policy_definition.get(top_level_logical_op)
        if not nested_definition:
            return top_level_predicament

        nested_logical_op = None
        for definition in nested_definition:
            for key, value in definition.items():
                if key in SUPPORTED_LOGICAL_OPERATORS:
                    nested_logical_op = key
                else:
                    predicate = self._create_predicate(key, value, record)
                    if predicate:
                        top_level_predicament.predicates.append(predicate)

        nested_predicament = None
        if nested_logical_op:
            nested_definition = nested_definition[0][nested_logical_op]
            if not isinstance(nested_definition, list):
                return None

            nested_predicament = Predicament(logical_op=nested_logical_op)
            for definition in nested_definition:
                for key, value in definition.items():
                    predicate = self._create_predicate(key, value, record)
                    if predicate:
                        nested_predicament.predicates.append(predicate)

        if nested_predicament:
            top_level_predicament.predicaments.append(nested_predicament)

        return top_level_predicament



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckResult
from checkov.github_actions.checks.base_github_action_check import BaseGithubActionsCheck
from checkov.yaml_doc.enums import BlockType


class EmptyWorkflowDispatch(BaseGithubActionsCheck):
    def __init__(self) -> None:
        name = "The build output cannot be affected by user parameters other than the build entry point and the " \
               "top-level source location. GitHub Actions workflow_dispatch inputs MUST be empty. "
        id = "CKV_GHA_7"
        super().__init__(
            name=name,
            id=id,
            block_type=BlockType.OBJECT,
            supported_entities=('on',)
        )

    def scan_conf(self, conf: dict[str, Any]) -> tuple[CheckResult, dict[str, Any]]:
        if isinstance(conf, list):
            for sub_conf in conf:
                if sub_conf == "workflow_dispatch":
                    return CheckResult.PASSED, sub_conf
            return CheckResult.UNKNOWN, {}

        if isinstance(conf, str):
            if conf == "workflow_dispatch":
                return CheckResult.PASSED, conf
            else:
                return CheckResult.UNKNOWN, conf

        workflow_dispatch = conf.get("workflow_dispatch")
        if isinstance(workflow_dispatch, dict):
            workflow_dispatch_inputs = workflow_dispatch.get("inputs", {})
            if workflow_dispatch_inputs:
                return CheckResult.FAILED, workflow_dispatch_inputs
        return CheckResult.PASSED, conf


check = EmptyWorkflowDispatch()



from __future__ import annotations

import re
from typing import TYPE_CHECKING, Any

from checkov.common.images.image_referencer import ImageReferencer, Image
from checkov.common.output.report import CheckType
from checkov.common.util.file_utils import read_file_with_any_encoding
from checkov.yaml_doc.runner import Runner as YamlRunner

# Import of the checks registry for a specific resource type
from checkov.argo_workflows.checks.registry import registry as template_registry

if TYPE_CHECKING:
    from checkov.common.checks.base_check_registry import BaseCheckRegistry

API_VERSION_PATTERN = re.compile(r"^apiVersion:\s*argoproj.io/", re.MULTILINE)
KIND_PATTERN = re.compile(r"^kind:\s*Workflow", re.MULTILINE)


class Runner(YamlRunner, ImageReferencer):
    check_type = CheckType.ARGO_WORKFLOWS  # noqa: CCE003  # a static attribute

    block_type_registries = {  # noqa: CCE003  # a static attribute
        "template": template_registry,
    }

    def require_external_checks(self) -> bool:
        return False

    def import_registry(self) -> BaseCheckRegistry:
        return self.block_type_registries["template"]

    def _parse_file(
        self, f: str, file_content: str | None = None
    ) -> tuple[dict[str, Any] | list[dict[str, Any]], list[tuple[int, str]]] | None:
        content = self._get_workflow_file_content(file_path=f)
        if content:
            return super()._parse_file(f=f, file_content=content)

        return None

    def _get_workflow_file_content(self, file_path: str) -> str | None:
        if not file_path.endswith((".yaml", ".yml")):
            return None

        content = read_file_with_any_encoding(file_path=file_path)
        if "argoproj.io" not in content:
            # the following regex will search more precisely, but no need to further process
            return None

        match_api = re.search(API_VERSION_PATTERN, content)
        if match_api:
            match_kind = re.search(KIND_PATTERN, content)
            if match_kind:
                # only scan Argo Workflows
                return content

        return None

    def is_workflow_file(self, file_path: str) -> bool:
        return self._get_workflow_file_content(file_path=file_path) is not None

    def get_images(self, file_path: str) -> set[Image]:
        """Get container images mentioned in a file

        Argo Workflows file can have a job and services run within a container.

        in the following sample file we can see a node:14.16 image:

        apiVersion: argoproj.io/v1alpha1
        kind: Workflow
        metadata:
          generateName: template-defaults-
        spec:
          entrypoint: main
          templates:
            - name: main
              steps:
                - - name: retry-backoff
                    template: retry-backoff
                - - name: whalesay
                    template: whalesay

            - name: whalesay
              container:
                image: argoproj/argosay:v2
                command: [cowsay]
                args: ["hello world"]

            - name: retry-backoff
              container:
                image: python:alpine3.6
                command: ["python", -c]
                # fail with a 66% probability
                args: ["import random; import sys; exit_code = random.choice([0, 1, 1]); sys.exit(exit_code)"]

        Source: https://github.com/argoproj/argo-workflows/blob/master/examples/template-defaults.yaml

        :return: List of container image short ids mentioned in the file.
        Example return value for a file with node:14.16 image: ['sha256:6a353e22ce']
        """

        images: set[Image] = set()
        parsed_file = self._parse_file(file_path)

        if not parsed_file:
            return images

        workflow, workflow_line_numbers = parsed_file

        if not isinstance(workflow, dict):
            # make type checking happy
            return images

        spec = workflow.get("spec")
        if spec:
            templates = spec.get("templates")
            if isinstance(templates, list):
                for template in templates:
                    container = template.get("container")
                    if container:
                        image = self.extract_image(file_path=file_path, container=container)
                        if image:
                            images.add(image)
                    script = template.get("script")
                    if script:
                        image = self.extract_image(file_path=file_path, container=script)
                        if image:
                            images.add(image)

        return images

    def extract_image(self, file_path: str, container: dict[str, Any]) -> Image | None:
        image_name = container.get("image")
        if image_name and isinstance(image_name, str):
            start_line = container.get("__startline__", 0)
            end_line = container.get("__endline__", 0)
            return Image(
                file_path=file_path,
                name=image_name,
                start_line=start_line,
                end_line=end_line,
            )

        return None



from typing import Any

from checkov.common.models.consts import ANY_VALUE
from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class DBInstanceLogging(BaseResourceValueCheck):
    def __init__(self) -> None:
        """
        NIST.800-53.r5 AC-2(4), NIST.800-53.r5 AC-4(26), NIST.800-53.r5 AC-6(9), NIST.800-53.r5 AU-10,
        NIST.800-53.r5 AU-12, NIST.800-53.r5 AU-2, NIST.800-53.r5 AU-3, NIST.800-53.r5 AU-6(3), NIST.800-53.r5 AU-6(4),
        NIST.800-53.r5 CA-7, NIST.800-53.r5 SC-7(10), NIST.800-53.r5 SC-7(9), NIST.800-53.r5 SI-3(8),
        NIST.800-53.r5 SI-4(20), NIST.800-53.r5 SI-7(8)
        Database logging should be enabled
        """
        name = "Ensure that RDS Cluster log capture is enabled"
        id = "CKV_AWS_324"
        supported_resources = ("aws_rds_cluster",)
        categories = (CheckCategories.LOGGING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "enabled_cloudwatch_logs_exports/[0]"

    def get_expected_value(self) -> Any:
        return ANY_VALUE


check = DBInstanceLogging()



from __future__ import annotations

from pathlib import Path
from typing import TYPE_CHECKING, Callable

from checkov.common.bridgecrew.check_type import CheckType
from checkov.github.dal import Github, CKV_METADATA
from checkov.json_doc.runner import Runner as JsonRunner
from checkov.runner_filter import RunnerFilter

if TYPE_CHECKING:
    from checkov.common.checks.base_check_registry import BaseCheckRegistry
    from checkov.common.output.report import Report


class Runner(JsonRunner):
    check_type = CheckType.GITHUB_CONFIGURATION  # noqa: CCE003  # a static attribute

    def __init__(self) -> None:
        self.github = Github()
        super().__init__()

    def run(
        self,
        root_folder: str | None = None,
        external_checks_dir: list[str] | None = None,
        files: list[str] | None = None,
        runner_filter: RunnerFilter | None = None,
        collect_skip_comments: bool = True
    ) -> Report | list[Report]:
        runner_filter = runner_filter or RunnerFilter()
        if not runner_filter.show_progress_bar:
            self.pbar.turn_off_progress_bar()

        self.prepare_data()

        report = super().run(
            root_folder=self.github.github_conf_dir_path,
            external_checks_dir=external_checks_dir,
            files=None,  # ignore file scans
            runner_filter=runner_filter,
            collect_skip_comments=collect_skip_comments,
        )
        JsonRunner._change_files_path_to_relative(report)  # type:ignore[arg-type]  # report can only be of type Report, not a list
        return report

    def prepare_data(self) -> None:
        self.github.persist_all_confs()

    def require_external_checks(self) -> bool:
        # default json runner require only external checks. Github runner brings build in checks
        return False

    def import_registry(self) -> BaseCheckRegistry:
        from checkov.github.registry import registry
        return registry

    def _load_files(
            self,
            files_to_load: list[str],
            filename_fn: Callable[[str], str] | None = None,
    ) -> None:
        super(Runner, self)._load_files(files_to_load=files_to_load, filename_fn=filename_fn)

        for file_path, definition in self.definitions.items():
            file_name = Path(file_path).stem
            ckv_metadata = {
                'file_name': file_name,
                'org_complementary_metadata': self.github.org_complementary_metadata,
                'repo_complementary_metadata': self.github.repo_complementary_metadata,
            }
            if isinstance(definition, dict):
                definition[CKV_METADATA] = ckv_metadata
            elif isinstance(definition, list):
                definition.append(ckv_metadata)



from typing import Optional, Any, Dict

from checkov.common.graph.checks_infra.enums import Operators
from checkov.common.checks_infra.solvers.attribute_solvers.number_of_words_equals_attribute_solver import NumberOfWordsEqualsAttributeSolver


class NumberOfWordsNotEqualsAttributeSolver(NumberOfWordsEqualsAttributeSolver):
    operator = Operators.NUMBER_OF_WORDS_NOT_EQUALS  # noqa: CCE003  # a static attribute

    def _get_operation(self, vertex: Dict[str, Any], attribute: Optional[str]) -> bool:
        return not super()._get_operation(vertex, attribute)
